@@ -14,8 +14,24 @@
 # ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 # OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
+"""
+Loaders are a simple way to manage design docs in your Python application. 
+Loaders are compatible with couchapp script (http://github.com/couchapp/couchapp).
+So it means that you can simply use couchdbkit as replacement for yours python
+applications with advantages of couchdbkit client. Compatible with couchapp means that
+you can also use macros to include javascript code or design doc members in your views,
+shows & lists.
+
+Current loaders are FileSystemDocsLoader and FileSystemDocLoader. The first
+one take a directory and retrieve all design docs before sending them to
+CouchDB. Second allow you to send only one design doc.
+"""
+from __future__ import with_statement
+
+import base64
 import copy
 import httplib
+import mimetypes
 import os
 import socket
 import sys
@@ -53,7 +69,7 @@ class BaseDocsLoader(object):
     def get_docs(self):
         raise NotImplementedError
     
-    def sync(self, dbs, verbose=False):
+    def sync(self, dbs, atomic=True, verbose=False):
         if not isinstance(dbs, (list, tuple)):
             dbs = [dbs]
         
@@ -88,9 +104,14 @@ class BaseDocsLoader(object):
                     if '_rev' in current:
                         new_doc['_rev'] = current.get('_rev')
                         
-                db[docid] = new_doc
-                if docid.startswith('_design/'):
-                    self.send_attachments(db, doc, verbose=verbose)
+                if not atomic:
+                    db[docid] = new_doc
+                    if docid.startswith('_design/'):
+                        self.send_attachments(db, doc, verbose=verbose)
+                else:
+                    if docid.startswith('_design/'):
+                        self.encode_attachments(db, doc, new_doc, verbose=verbose)
+                    db[docid] = new_doc
             
     def _put_attachment(self, db, doc, content, filename, content_length=None, 
             verbose=False):
@@ -116,6 +137,54 @@ class BaseDocsLoader(object):
                     print >>sys.stderr, "%s file not uploaded, sorry." % filename
                 break
             
+    def encode_attachments(self, db, design_doc,new_doc, verbose=False):
+        # init vars
+        all_signatures = {}                  
+        if not 'couchapp' in design_doc:
+            design_doc['couchapp'] = {}
+        
+        _signatures = design_doc['couchapp'].get('signatures', {})
+        _length = design_doc['couchapp'].get('length', {})
+        _attachments = design_doc.get('_attachments', {})
+        docid = design_doc['_id']
+        
+        attachments = _attachments.copy()
+        current_design = {}
+        try:
+            current_design = db[docid]
+        except ResourceNotFound:
+            pass
+            
+        new_attachments = current_design.get('_attachments', {})
+        metadata = current_design.get('couchapp', {})
+        if 'signatures' in metadata:
+            all_signatures = metadata['signatures'].copy()
+            for filename in metadata['signatures'].iterkeys():
+                if not filename not in _signatures:
+                    del new_attachments[filename]
+                    del all_signatures[filename]
+                elif _signatures[filename] == metadata['signatures'][filename]:
+                    del attachments[filename]
+
+        for filename, value in attachments.iteritems():
+            content_length = _length.get(filename, None)
+            if verbose:
+                print "Attaching %s (%s)" % (filename, content_length)
+            
+            with open(value, "rb") as f:
+                new_attachments[filename] = {
+                    "content_type": ';'.join(filter(None, mimetypes.guess_type(filename))),
+                    "data": base64.b64encode(f.read()),
+                }
+                     
+        # update signatures
+        if not 'couchapp' in new_doc:
+            new_doc['couchapp'] = {}
+
+        all_signatures.update(_signatures)
+        new_doc['couchapp'].update({'signatures': _signatures})
+        new_doc['_attachments'] = new_attachments
+                
     def send_attachments(self, db, design_doc, verbose=False):
         # init vars
         all_signatures = {}                  
@@ -141,12 +210,15 @@ class BaseDocsLoader(object):
                     del attachments[filename]
 
         for filename, value in attachments.iteritems():
+            content_length = _length.get(filename, None)
             if verbose:
-                print "Attaching %s" % filename
-            # fix issue with httplib that raises BadStatusLine
-            # error because it didn't close the connection
-            self._put_attachment(db, current_design, value, filename, 
-                    content_length=_length.get(filename, None), verbose=verbose)
+                print "Attaching %s (%s)" % (filename, content_length)
+            
+            with open(value, "rb") as f:
+                # fix issue with httplib that raises BadStatusLine
+                # error because it didn't close the connection
+                self._put_attachment(db, current_design, f, filename, 
+                    content_length=content_length, verbose=verbose)
                      
         # update signatures
         current_design = db[docid]
@@ -156,10 +228,9 @@ class BaseDocsLoader(object):
         all_signatures.update(_signatures)
         current_design['couchapp'].update({'signatures': all_signatures})
         db[docid] = current_design
-                
 
 class FileSystemDocsLoader(BaseDocsLoader):
-    """ Load ocs from the filesystem. This loader can find docs
+    """ Load docs from the filesystem. This loader can find docs
     in folders on the file system and is the preferred way to load them. 
     
     The loader takes the path for design docs as a string  or if multiple
@@ -220,45 +291,47 @@ class FileSystemDocsLoader(BaseDocsLoader):
             if not os.path.isdir(designpath):
                 raise DocsPathNotFound("%s doesn't exist" % designpath)
             for name in os.listdir(designpath):
-                if name.startswith('.'):
-                    continue
-                elif os.path.isfile(name):
-                    continue
-                else:
-                    design_doc = {}
-                    manifest = []
-                    objects = {}
-                    docid = design_doc['_id'] = "_design/%s" % name
-                    app_dir = os.path.join(designpath, name)
-                    attach_dir = os.path.join(app_dir, '_attachments')
-
-                    design_doc.update(self.dir_to_fields(app_dir, manifest=manifest,
-                            verbose=verbose))
-
-                    if not 'couchapp' in design_doc:
-                        design_doc['couchapp'] = {}
-
-                    if 'shows' in design_doc:
-                        package_shows(design_doc, design_doc['shows'], app_dir, objects, verbose=verbose)
-
-                    if 'lists' in design_doc:
-                        package_shows(design_doc, design_doc['lists'], app_dir, objects, verbose=verbose)
-
-                    if 'views' in design_doc:
-                        package_views(design_doc, design_doc["views"], app_dir, objects, verbose=verbose)
-
-                    couchapp = design_doc.get('couchapp', False)
-                    design_doc.update({
-                        'couchapp': {
-                            'manifest': manifest,
-                            'objects': objects
-                        }
-                    })
-                    self.attach(design_doc, attach_dir, docid, verbose=verbose)
-                    self.attach_vendors(design_doc, app_dir, docid, verbose=verbose)
-                    docs.append(design_doc)
-        return docs
+                docs.append(self.get_designdoc(designpath, name, verbose=verbose))
                 
+        return docs
+        
+    def get_designdoc(self, root, name, design_name=None, verbose=False):
+        if not name.startswith('.') and not os.path.isfile(name):
+            design_doc = {}
+            manifest = []
+            objects = {}
+            if design_name is None:
+                docid = design_doc['_id'] = "_design/%s" % name
+            else:
+                docid = design_doc['_id'] = "_design/%s" % design_name
+            app_dir = os.path.join(root, name)
+            attach_dir = os.path.join(app_dir, '_attachments')
+
+            design_doc.update(self.dir_to_fields(app_dir, manifest=manifest,
+                    verbose=verbose))
+
+            if not 'couchapp' in design_doc:
+                design_doc['couchapp'] = {}
+
+            if 'shows' in design_doc:
+                package_shows(design_doc, design_doc['shows'], app_dir, objects, verbose=verbose)
+
+            if 'lists' in design_doc:
+                package_shows(design_doc, design_doc['lists'], app_dir, objects, verbose=verbose)
+
+            if 'views' in design_doc:
+                package_views(design_doc, design_doc["views"], app_dir, objects, verbose=verbose)
+
+            couchapp = design_doc.get('couchapp', False)
+            design_doc.update({
+                'couchapp': {
+                    'manifest': manifest,
+                    'objects': objects
+                }
+            })
+            self.attach(design_doc, attach_dir, docid, verbose=verbose)
+            self.attach_vendors(design_doc, app_dir, docid, verbose=verbose)
+        return design_doc
             
     def dir_to_fields(self, app_dir, current_dir='', depth=0,
             manifest=[], verbose=False):
@@ -309,7 +382,16 @@ class FileSystemDocsLoader(BaseDocsLoader):
                 try:
                     content = read_file(current_path)
                 except UnicodeDecodeError, e:
-                    print >>sys.stderr, str(e)
+                    if verbose >= 2:
+                        print >>sys.stderr, "%s isn't encoded in utf8" % current_path
+                    content = self.ui.read(current_path, utf8=False)
+                    try:
+                        content.encode('utf-8')
+                    except UnicodeError, e:
+                        print >>sys.stderr, "plan B didn't work, %s is a binary" % current_path
+                        print >>sys.stderr, "use plan C: encode to base64"
+                        content = "base64-encoded;%s" % base64.b64encode(content)
+                        
                 if name.endswith('.json'):
                     try:
                         content = json.loads(content)
@@ -360,7 +442,7 @@ class FileSystemDocsLoader(BaseDocsLoader):
                         if vendor is not None:
                             name = os.path.join('vendor/%s' % vendor, name)
                         _signatures[name] = sign_file(file_path)
-                        _attachments[name] = open(file_path, 'rb')
+                        _attachments[name] = file_path
                         _length[name] = int(os.path.getsize(file_path))
         
         for prop in ('couchapp', '_attachments'):
@@ -375,4 +457,19 @@ class FileSystemDocsLoader(BaseDocsLoader):
             
         doc['_attachments'].update(_attachments)
         doc['couchapp']['signatures'].update(_signatures)
-        doc['couchapp']['length'].update(_length)
\ No newline at end of file
+        doc['couchapp']['length'].update(_length)
+        
+class FileSystemDocLoader(FileSystemDocsLoader):
+    """ Load only one design doc from a path on the filesystem.
+        
+        >>> loader = FileSystemDocLoader("/path/to/designdocfolder", "nameodesigndoc")
+    """
+    
+    def __init__(self, designpath, name, design_name=None):
+        self.designpath = designpath
+        self.name = name
+        self.design_name = design_name
+        
+    def get_docs(self, verbose=False):
+        return [self.get_designdoc(self.designpath, self.name, 
+                design_name=self.design_name, verbose=verbose)]
\ No newline at end of file
