@@ -16,7 +16,8 @@
 
 """ module that provide a Document object that allow you 
 to map statically, dynamically or both a CouchDB
-document in python """
+document in python 
+"""
 
 import datetime
 import decimal
@@ -38,13 +39,18 @@ __all__ = ['ReservedWordError', 'ALLOWED_PROPERTY_TYPES', 'DocumentSchema',
 
 _RESERVED_WORDS = ['_id', '_rev', '$schema', 'type']
 
-_NODOC_WORDS = ['doc_type','id', 'rev', 'type']
+_NODOC_WORDS = ['doc_type', 'type']
 
 def check_reserved_words(attr_name):
     if attr_name in _RESERVED_WORDS:
         raise ReservedWordError(
             "Cannot define property using reserved word '%(attr_name)s'." % 
             locals())
+            
+def valid_id(value):
+    if isinstance(value, basestring) and not value.startswith('_'):
+        return value
+    raise TypeError('id "%s" is invalid' % value)
 
 class SchemaProperties(type):
 
@@ -173,39 +179,44 @@ class DocumentSchema(object):
         start with '_') a,d add it to `_dynamic_properties` dict. If value is 
         a list or a dict we use LazyList and LazyDict to maintain in the value.
         """
-        check_reserved_words(key)
-        if not hasattr( self, key ) and not self._allow_dynamic_properties:
-            raise AttributeError("%s is not defined in schema (not a valid property)" % key)
-            
-        if not key.startswith('_') and \
-                key not in self.properties() and \
-                key not in dir(self): 
-            if type(value) not in ALLOWED_PROPERTY_TYPES and \
-                    not isinstance(value, (p.Property,)):
-                raise TypeError("Document Schema cannot accept values of type '%s'." %
-                        type(value).__name__)
-                        
-            if self._dynamic_properties is None:
-                self._dynamic_properties = {}
-                
-            if isinstance(value, dict):
-                if key not in self._doc:
-                    self._doc[key] = {}
-                value = LazyDict(value, self._doc[key])
-            elif isinstance(value, list):
-                if key not in self._doc:
-                    self._doc[key] = []
-                value = LazyList(value, self._doc[key])
-            self._dynamic_properties[key] = value
-
-            if not isinstance(value, (p.Property,)) and \
-                    not isinstance(value, dict) and \
-                    not isinstance(value, list):
-                if callable(value):
-                    value = value()
-                self._doc[key] = convert_property(value)
+        
+        if key == "_id" and valid_id(value):
+            self._doc['_id'] = value
         else:
-            object.__setattr__(self, key, value)
+            check_reserved_words(key)
+            if not hasattr( self, key ) and not self._allow_dynamic_properties:
+                raise AttributeError("%s is not defined in schema (not a valid property)" % key)
+            
+        
+            elif not key.startswith('_') and \
+                    key not in self.properties() and \
+                    key not in dir(self): 
+                if type(value) not in ALLOWED_PROPERTY_TYPES and \
+                        not isinstance(value, (p.Property,)):
+                    raise TypeError("Document Schema cannot accept values of type '%s'." %
+                            type(value).__name__)
+                        
+                if self._dynamic_properties is None:
+                    self._dynamic_properties = {}
+                
+                if isinstance(value, dict):
+                    if key not in self._doc:
+                        self._doc[key] = {}
+                    value = LazyDict(value, self._doc[key])
+                elif isinstance(value, list):
+                    if key not in self._doc:
+                        self._doc[key] = []
+                    value = LazyList(value, self._doc[key])
+                self._dynamic_properties[key] = value
+
+                if not isinstance(value, (p.Property,)) and \
+                        not isinstance(value, dict) and \
+                        not isinstance(value, list):
+                    if callable(value):
+                        value = value()
+                    self._doc[key] = convert_property(value)
+            else:
+                object.__setattr__(self, key, value)
 
     def __delattr__(self, key):
         """ delete property
@@ -223,6 +234,8 @@ class DocumentSchema(object):
         """
         if self._dynamic_properties and key in self._dynamic_properties:
             return self._dynamic_properties[key]
+        elif key == "_id" or key == "_rev":
+            return self._doc.get(key)
         return getattr(super(DocumentSchema, self), key)
     
     def __getitem__(self, key):
@@ -233,17 +246,13 @@ class DocumentSchema(object):
         except AttributeError, e:
             if key in self._doc:
                 return self._doc[key]
-            raise KeyError, e
-        except:
             raise
 
     def __setitem__(self, key, value):
         """ add a property
         """
-        try:
-            setattr(self, key, value)
-        except AttributeError, e:
-            raise KeyError, e
+        setattr(self, key, value)
+    
 
     def __delitem__(self, key):
         """ delete a property
@@ -257,9 +266,9 @@ class DocumentSchema(object):
     def __contains__(self, key):
         """ does object contain this propery ?
 
-        :param key: name of property
+        @param key: name of property
         
-        :return: True if key exist.
+        @return: True if key exist.
         """ 
         if key in self.all_properties():
             return True
@@ -392,20 +401,12 @@ class DocumentBase(DocumentSchema):
     _db = None
 
     def __init__(self, d=None, **kwargs):
-        if 'id' in kwargs:
-            self._doc['_id'] = kwargs.pop('_id')
-        DocumentSchema.__init__(self, d, **kwargs) 
-    
-    def _get_id(self):
-        return self._doc.get('_id', None)
-
-    def _set_id(self, docid):
-        if not isinstance(docid, basestring):
-            raise TypeError('doc id should be a string')
-        self._doc['_id'] = docid
-    id = property(_get_id, _set_id)
-
-    rev = property(lambda self: self._doc.get('_rev'))
+        docid = None
+        if '_id' in kwargs:
+            docid = kwargs.pop('_id')
+        DocumentSchema.__init__(self, d, **kwargs)
+        if docid is not None:
+            self._doc['_id'] = valid_id(docid)
 
     @classmethod
     def set_db(cls, db):
@@ -423,7 +424,7 @@ class DocumentBase(DocumentSchema):
     def save(self):
         """ Save document in database.
             
-        :params db: couchdbkit.core.Database instance
+        @params db: couchdbkit.core.Database instance
         """
         self.validate()    
         if self._db is None:
@@ -439,7 +440,7 @@ class DocumentBase(DocumentSchema):
     def bulk_save(cls, docs):
         """ Save multiple documents in database.
             
-        :params docs: list of couchdbkit.schema.Document instance
+        @params docs: list of couchdbkit.schema.Document instance
         """
         if cls._db is None:
             raise TypeError("doc database required to save document")
@@ -495,13 +496,13 @@ class AttachmentMixin(object):
         content_type=None, content_length=None):
         """ Add attachement to a document.
  
-        :param content: string or :obj:`File` object.
-        :param name: name or attachment (file name).
-        :param content_type: string, mimetype of attachment.
+        @param content: string or :obj:`File` object.
+        @param name: name or attachment (file name).
+        @param content_type: string, mimetype of attachment.
         If you don't set it, it will be autodetected.
-        :param content_lenght: int, size of attachment.
+        @param content_lenght: int, size of attachment.
 
-        :return: bool, True if everything was ok.
+        @return: bool, True if everything was ok.
         """
         if not hasattr(self, '_db'):
             raise TypeError("doc database required to save document")
@@ -511,9 +512,9 @@ class AttachmentMixin(object):
     def delete_attachment(self, name):
         """ delete attachement of documen
         
-        :param name: name of attachement
+        @param name: name of attachement
     
-        :return: dict, withm member ok setto True if delete was ok.
+        @return: dict, withm member ok setto True if delete was ok.
         """
         if not hasattr(self, '_db'):
             raise TypeError("doc database required to save document")
@@ -522,9 +523,9 @@ class AttachmentMixin(object):
     def fetch_attachment(self, name):
         """ get attachment in document
         
-        :param name: name of attachment default: default result
+        @param name: name of attachment default: default result
 
-        :return: str or unicode, attachment
+        @return: str or unicode, attachment
         """
         if not hasattr(self, '_db'):
             raise TypeError("doc database required to save document")
@@ -578,10 +579,10 @@ class QueryMixin(object):
         Results of view are automatically wrapped
         to Document object.
 
-        :params view_name: str, name of view
-        :params params:  params of view
+        @params view_name: str, name of view
+        @params params:  params of view
 
-        :return: :class:`simplecouchdb.core.ViewResults` instance. All
+        @return: :class:`simplecouchdb.core.ViewResults` instance. All
         results are wrapped to current document instance.
         """
         return cls.__view(view_type="view", data=view_name, wrapper=wrapper, 
@@ -594,10 +595,10 @@ class QueryMixin(object):
         results are automatically wrapped to 
         Document object.
 
-        :params design: design object, See `simplecouchd.client.Database`
-        :params params:  params of view
+        @params design: design object, See `simplecouchd.client.Database`
+        @params params:  params of view
 
-        :return: Like view, return a :class:`simplecouchdb.core.ViewResults` 
+        @return: Like view, return a :class:`simplecouchdb.core.ViewResults` 
         instance. All results are wrapped to current document instance.
         """
         return cls.__view(view_type="temp_view", data=design, wrapper=wrapper, 
